@@ -61,8 +61,9 @@ from .enums.ErrorCode import ErrorCode
 from .GXDLMSTranslatorStructure import GXDLMSTranslatorStructure
 from .enums.RequestTypes import RequestTypes
 from .SerialnumberCounter import SerialNumberCounter
+from ._GXObjectFactory import _GXObjectFactory
 
-#pylint: disable=bad-option-value,too-many-instance-attributes,too-many-arguments,too-many-public-methods,useless-object-inheritance
+#pylint:disable=bad-option-value,too-many-instance-attributes,too-many-arguments,too-many-public-methods,useless-object-inheritance
 class GXDLMSClient(object):
     """
     GXDLMS implements methods to communicate with DLMS/COSEM metering devices.
@@ -167,7 +168,8 @@ class GXDLMSClient(object):
 
     #
     # Source system title.
-    # Meter returns system title when ciphered connection is made or GMAC authentication is used.
+    # Meter returns system title when ciphered connection is made or GMAC
+    # authentication is used.
     #
     sourceSystemTitle = property(__getSourceSystemTitle, None)
 
@@ -287,10 +289,13 @@ class GXDLMSClient(object):
     def __getNegotiatedConformance(self):
         return self.settings.negotiatedConformance
 
+    def __setNegotiatedConformance(self, value):
+        self.settings.negotiatedConformance = value
+
     #
     # Functionality what server offers.
     #
-    negotiatedConformance = property(__getNegotiatedConformance)
+    negotiatedConformance = property(__getNegotiatedConformance, __setNegotiatedConformance)
 
 
     def __getProposedConformance(self):
@@ -570,7 +575,7 @@ class GXDLMSClient(object):
         info = _GXDataInfo()
         equals = False
         ic = 0
-        value = _GXCommon.getData(reply, info)
+        value = _GXCommon.getData(self.settings, reply, info)
         if value:
             if self.settings.authentication == Authentication.HIGH_ECDSA:
                 raise ValueError("ECDSA is not supported.")
@@ -641,7 +646,7 @@ class GXDLMSClient(object):
         cls.__updateObjectData(obj, type_, version, baseName, ln, accessRights)
         return obj
 
-    def parseSNObjects(self, buff, onlyKnownObjects):
+    def parseSNObjects(self, buff, onlyKnownObjects, ignoreInactiveObjects):
         # pylint: disable=unidiomatic-typecheck
         buff.position = 0
         size = buff.getUInt8()
@@ -657,17 +662,17 @@ class GXDLMSClient(object):
             info.count = 0
             info.index = 0
             info.type_ = DataType.NONE
-            objects = _GXCommon.getData(buff, info)
+            objects = _GXCommon.getData(self.settings, buff, info)
             if len(objects) != 4:
                 raise Exception("Invalid structure format.")
             classID = objects[1]
-            baseName = objects[0]
-            if baseName > 0:
-                comp = self.__createDLMSObject(classID, objects[2], baseName, objects[3], None)
-                if not onlyKnownObjects or type(comp) != GXDLMSObject:
+            baseName = int(objects[0]) & 0xFFFF
+            comp = self.__createDLMSObject(classID, objects[2], baseName, objects[3], None)
+            if (not onlyKnownObjects or type(comp) != GXDLMSObject):
+                if not ignoreInactiveObjects or comp.logicalName != "0.0.127.0.0.0":
                     items.append(comp)
-                else:
-                    print("Unknown object : " + str(classID) + " " + str(baseName))
+            else:
+                print("Unknown object : " + str(classID) + " " + str(baseName))
             objPos += 1
         return items
 
@@ -692,26 +697,26 @@ class GXDLMSClient(object):
                     tmp = methodAccess[1]
                 obj.setMethodAccess(id_, tmp)
         if baseName is not None:
-            obj.shortName = baseName
+            obj.shortName = int(baseName)
         if version is not None:
-            obj.version = version
+            obj.version = int(version)
         obj.logicalName = _GXCommon.toLogicalName(logicalName)
 
-    def parseObjects(self, data, onlyKnownObjects=True):
+    def parseObjects(self, data, onlyKnownObjects=True, ignoreInactiveObjects=True):
         if not data:
             raise Exception("Invalid parameter.")
         objects = None
         if self.useLogicalNameReferencing:
-            objects = self.parseLNObjects(data, onlyKnownObjects)
+            objects = self.parseLNObjects(data, onlyKnownObjects, ignoreInactiveObjects)
         else:
-            objects = self.parseSNObjects(data, onlyKnownObjects)
+            objects = self.parseSNObjects(data, onlyKnownObjects, ignoreInactiveObjects)
         self.settings.objects = objects
 
         c = GXDLMSConverter(self.standard)
         c.updateOBISCodeInformation(objects)
         return objects
 
-    def parseLNObjects(self, buff, onlyKnownObjects):
+    def parseLNObjects(self, buff, onlyKnownObjects, ignoreInactiveObjects):
         # pylint: disable=unidiomatic-typecheck
         size = buff.getInt8()
         if size != 0x01:
@@ -726,14 +731,15 @@ class GXDLMSClient(object):
             info.type_ = DataType.NONE
             info.index = 0
             info.count = 0
-            objects = _GXCommon.getData(buff, info)
+            objects = _GXCommon.getData(self.settings, buff, info)
             if len(objects) != 4:
                 raise Exception("Invalid structure format.")
             classID = objects[0]
             if classID > 0:
                 comp = self.__createDLMSObject(classID, objects[1], 0, objects[2], objects[3])
-                if not onlyKnownObjects or type(comp) != GXDLMSObject:
-                    items.append(comp)
+                if (not onlyKnownObjects or type(comp) != GXDLMSObject):
+                    if not ignoreInactiveObjects or comp.logicalName != "0.0.127.0.0.0":
+                        items.append(comp)
                 else:
                     print("Unknown object : " + str(classID) + " " + _GXCommon.toLogicalName(objects[2]))
             objPos += 1
@@ -746,16 +752,18 @@ class GXDLMSClient(object):
                 type_ = DataType.DATE
                 target.setUIDataType(attributeIndex, type_)
             if type_ != DataType.NONE:
-                value = self.changeType(value, type_)
+                value = self.changeType(value, type_, self.settings.useUtc2NormalTime)
         e = ValueEventArgs(self.settings, target, attributeIndex, 0, parameters)
         e.value = value
         target.setValue(self.settings, e)
         return target.getValues()[attributeIndex - 1]
 
     @classmethod
-    def getValue(cls, data):
+    def getValue(cls, data, useUtc):
+        settings = GXDLMSSettings(False)
+        settings.useUtc2NormalTime = useUtc
         info = _GXDataInfo()
-        return _GXCommon.getData(data, info)
+        return _GXCommon.getData(settings, data, info)
 
 
     def updateValues(self, list_, values):
@@ -767,8 +775,10 @@ class GXDLMSClient(object):
             pos += 1
 
     @classmethod
-    def changeType(cls, value, type_):
-        return _GXCommon.changeType(value, type_)
+    def changeType(cls, value, type_, useUtc=False):
+        settings = GXDLMSSettings(False)
+        settings.useUtc2NormalTime = useUtc
+        return _GXCommon.changeType(settings, value, type_)
 
     def getObjectsRequest(self):
         #pylint: disable=bad-option-value,redefined-variable-type
@@ -784,7 +794,8 @@ class GXDLMSClient(object):
         return self.__method(item.name, item.objectType, index, data, type_)
 
     def __method(self, name, objectType, methodIndex, value, dataType=DataType.NONE):
-        #pylint: disable=bad-option-value,redefined-variable-type,too-many-locals
+        #pylint:
+        #disable=bad-option-value,redefined-variable-type,too-many-locals
         if not name or methodIndex < 1:
             raise ValueError("Invalid parameter")
         self.settings.resetBlockIndex()
@@ -797,7 +808,7 @@ class GXDLMSClient(object):
         reply = None
         data = GXByteBuffer()
         attributeDescriptor = GXByteBuffer()
-        _GXCommon.setData(data, type_, value)
+        _GXCommon.setData(self.settings, data, type_, value)
         if self.useLogicalNameReferencing:
             attributeDescriptor.setUInt16(objectType)
             attributeDescriptor.set(_GXCommon.logicalNameToBytes(str(name)))
@@ -850,7 +861,7 @@ class GXDLMSClient(object):
         reply = None
         data = GXByteBuffer()
         attributeDescriptor = GXByteBuffer()
-        _GXCommon.setData(data, type_, value)
+        _GXCommon.setData(self.settings, data, type_, value)
         if self.useLogicalNameReferencing:
             attributeDescriptor.setUInt16(objectType)
             attributeDescriptor.set(_GXCommon.logicalNameToBytes(str(name)))
@@ -900,7 +911,7 @@ class GXDLMSClient(object):
                 type_ = it.target.getDataType(it.index)
                 if type_ == DataType.NONE:
                     raise Exception("Invalid parameter. In python value type must give.")
-            _GXCommon.setData(data, type_, value)
+            _GXCommon.setData(self.settings, data, type_, value)
         if self.useLogicalNameReferencing:
             p = GXDLMSLNParameters(self.settings, 0, Command.SET_REQUEST, SetRequestType.WITH_LIST, bb, data, 0xff)
             reply = GXDLMS.getLnMessages(p)
@@ -997,15 +1008,16 @@ class GXDLMSClient(object):
             raise ValueError("index")
         if count < 0:
             raise ValueError("count")
+        pg.buffer = list()
         buff = GXByteBuffer(19)
         buff.setUInt8(0x02)
         buff.setUInt8(DataType.STRUCTURE)
         buff.setUInt8(0x04)
-        _GXCommon.setData(buff, DataType.UINT32, index)
+        _GXCommon.setData(self.settings, buff, DataType.UINT32, index)
         if count == 0:
-            _GXCommon.setData(buff, DataType.UINT32, count)
+            _GXCommon.setData(self.settings, buff, DataType.UINT32, count)
         else:
-            _GXCommon.setData(buff, DataType.UINT32, index + count - 1)
+            _GXCommon.setData(self.settings, buff, DataType.UINT32, index + count - 1)
         columnIndex = 1
         columnCount = 0
         pos = 0
@@ -1027,11 +1039,12 @@ class GXDLMSClient(object):
                         break
                 if not found:
                     raise ValueError("Invalid column: " + c.logicalName)
-        _GXCommon.setData(buff, DataType.UINT16, columnIndex)
-        _GXCommon.setData(buff, DataType.UINT16, columnCount)
+        _GXCommon.setData(self.settings, buff, DataType.UINT16, columnIndex)
+        _GXCommon.setData(self.settings, buff, DataType.UINT16, columnCount)
         return self._read(pg.name, ObjectType.PROFILE_GENERIC, 2, buff)
 
     def readRowsByRange(self, pg, start, end, columns=None):
+        pg.buffer = list()
         self.settings.resetBlockIndex()
         if not isinstance(start, GXDateTime):
             start = GXDateTime(start)
@@ -1048,16 +1061,16 @@ class GXDLMSClient(object):
         buff.setUInt8(0x04)
         buff.setUInt8(DataType.STRUCTURE)
         buff.setUInt8(0x04)
-        _GXCommon.setData(buff, DataType.UINT16, sort.objectType)
-        _GXCommon.setData(buff, DataType.OCTET_STRING, _GXCommon.logicalNameToBytes(sort.logicalName))
-        _GXCommon.setData(buff, DataType.INT8, 2)
-        _GXCommon.setData(buff, DataType.UINT16, sort.version)
+        _GXCommon.setData(self.settings, buff, DataType.UINT16, sort.objectType)
+        _GXCommon.setData(self.settings, buff, DataType.OCTET_STRING, _GXCommon.logicalNameToBytes(sort.logicalName))
+        _GXCommon.setData(self.settings, buff, DataType.INT8, 2)
+        _GXCommon.setData(self.settings, buff, DataType.UINT16, sort.version)
         if pg.captureObjects and isinstance(pg.captureObjects[0][0], GXDLMSData) and pg.captureObjects[0][0].logicalName == "0.0.1.1.0.255":
-            _GXCommon.setData(buff, DataType.UINT32, GXDateTime.toUnixTime(start))
-            _GXCommon.setData(buff, DataType.UINT32, GXDateTime.toUnixTime(end))
+            _GXCommon.setData(self.settings, buff, DataType.UINT32, GXDateTime.toUnixTime(start))
+            _GXCommon.setData(self.settings, buff, DataType.UINT32, GXDateTime.toUnixTime(end))
         else:
-            _GXCommon.setData(buff, DataType.OCTET_STRING, start)
-            _GXCommon.setData(buff, DataType.OCTET_STRING, end)
+            _GXCommon.setData(self.settings, buff, DataType.OCTET_STRING, start)
+            _GXCommon.setData(self.settings, buff, DataType.OCTET_STRING, end)
         buff.setUInt8(DataType.ARRAY)
         if not columns:
             buff.setUInt8(0x00)
@@ -1066,15 +1079,14 @@ class GXDLMSClient(object):
             for it in columns:
                 buff.setUInt8(DataType.STRUCTURE)
                 buff.setUInt8(4)
-                _GXCommon.setData(buff, DataType.UINT16, it[0].objectType)
-                _GXCommon.setData(buff, DataType.OCTET_STRING, _GXCommon.logicalNameToBytes(it[0].logicalName))
-                _GXCommon.setData(buff, DataType.INT8, it[1].attributeIndex)
-                _GXCommon.setData(buff, DataType.INT16, it[1].dataIndex)
+                _GXCommon.setData(self.settings, buff, DataType.UINT16, it[0].objectType)
+                _GXCommon.setData(self.settings, buff, DataType.OCTET_STRING, _GXCommon.logicalNameToBytes(it[0].logicalName))
+                _GXCommon.setData(self.settings, buff, DataType.INT8, it[1].attributeIndex)
+                _GXCommon.setData(self.settings, buff, DataType.INT16, it[1].dataIndex)
         return self._read(pg.name, ObjectType.PROFILE_GENERIC, 2, buff)
 
     @classmethod
     def createObject(cls, type_):
-        from ._GXObjectFactory import _GXObjectFactory
         return _GXObjectFactory.createObject(type_)
 
     def receiverReady(self, type_):
@@ -1175,7 +1187,7 @@ class GXDLMSClient(object):
                 type_ = it.target.getDataType(it.index)
                 if type_ == DataType.NONE:
                     raise Exception("Invalid parameter. In python value type must give.")
-                _GXCommon.setData(bb, type_, value)
+                _GXCommon.setData(self.settings, bb, type_, value)
         p = GXDLMSLNParameters(self.settings, 0, Command.ACCESS_REQUEST, 0xFF, None, bb, 0xff)
         if time:
             p.time = GXDateTime(time)
@@ -1191,7 +1203,7 @@ class GXDLMSClient(object):
         reply = list(cnt)
         while pos != cnt:
             info.clear()
-            value = _GXCommon.getData(data, info)
+            value = _GXCommon.getData(self.settings, data, info)
             values_.append(value)
             pos += 1
         cnt = _GXCommon.getObjectCount(data)
