@@ -132,7 +132,7 @@ class _GXAPDU:
         data.setUInt8(0x5)
         data.setUInt8(0x8)
         data.setUInt8(0x1)
-        if settings.useLogicalNameReferencing:
+        if settings.getUseLogicalNameReferencing():
             if ciphered:
                 data.setUInt8(3)
             else:
@@ -184,7 +184,7 @@ class _GXAPDU:
             data.setUInt8(0x00)
         else:
             data.setUInt8(1)
-            _GXCommon.setObjectCount(len(settings.Cipher.DedicatedKey), data)
+            _GXCommon.setObjectCount(len(settings.cipher.dedicatedKey), data)
             data.set(settings.cipher.dedicatedKey)
         #  encoding of the response-allowed component (BOOLEAN DEFAULT TRUE)
         #  usage flag (FALSE, default value TRUE conveyed)
@@ -474,14 +474,14 @@ class _GXAPDU:
                     settings.setUseLogicalNameReferencing(True)
                 else:
                     #  If LN
-                    if not settings.useLogicalNameReferencing and xml is None:
+                    if not settings.getUseLogicalNameReferencing() and xml is None:
                         raise ValueError("Invalid VAA.")
             elif tag == 0xFA00:
                 #  If SN
                 if initiateRequest:
                     settings.setUseLogicalNameReferencing(False)
                 else:
-                    if settings.useLogicalNameReferencing:
+                    if settings.getUseLogicalNameReferencing():
                         raise ValueError("Invalid VAA.")
             else:
                 #  Unknown VAA.
@@ -599,7 +599,7 @@ class _GXAPDU:
             else:
                 return False
             return True
-        if settings.useLogicalNameReferencing:
+        if settings.getUseLogicalNameReferencing():
             if name == 1:
                 return True
             #  If ciphering is used.
@@ -897,7 +897,7 @@ class _GXAPDU:
                 xml.append(GXByteBuffer.hex(settings.getStoCChallenge(), False))
                 xml.append(TranslatorGeneralTags.CHAR_STRING, False)
                 xml.append(tag, False)
-                xml.append("\r\n")
+                xml.append("\n")
 
     @classmethod
     def appendClientSystemTitleToXml(cls, settings, xml):
@@ -981,7 +981,7 @@ class _GXAPDU:
         data.set(bb.subArray(1, 3))
         data.setUInt16(settings.maxPduSize)
         #  VAA Name VAA name (0x0007 for LN referencing and 0xFA00 for SN)
-        if settings.useLogicalNameReferencing:
+        if settings.getUseLogicalNameReferencing():
             data.setUInt16(0x0007)
         else:
             data.setUInt16(0xFA00)
