@@ -38,7 +38,7 @@ from ..internal._GXCommon import _GXCommon
 from ..GXByteBuffer import GXByteBuffer
 from ..GXDateTime import GXDateTime
 from ..enums import ObjectType, DataType
-from .enums import PaymentMode, AccountStatus, AccountCreditStatus, Currency, CreditCollectionConfiguration
+from .enums import PaymentMode, AccountStatus, AccountCreditStatus
 from .GXCurrency import GXCurrency
 from .GXTokenGatewayConfiguration import GXTokenGatewayConfiguration
 from .GXCreditChargeConfiguration import GXCreditChargeConfiguration
@@ -99,6 +99,18 @@ class GXDLMSAccount(GXDLMSObject, IGXDLMSBase):
                 self.maxProvision,
                 self.maxProvisionPeriod]
 
+    def activate(self, client):
+        """Activate account."""
+        return client.method(self.getName(), self.objectType, 1, 0, DataType.INT8)
+
+    def close(self, client):
+        """Close account."""
+        return client.method(self.getName(), self.objectType, 2, 0, DataType.INT8)
+
+    def reset(self, client):
+        """Reset account."""
+        return client.method(self.getName(), self.objectType, 3, 0, DataType.INT8)
+
     #
     # Returns collection of attributes to read.  If attribute is static
     #      and
@@ -288,7 +300,7 @@ class GXDLMSAccount(GXDLMSObject, IGXDLMSBase):
                     bb.setUInt8(DataType.OCTET_STRING)
                     bb.setUInt8(6)
                     bb.set(_GXCommon.logicalNameToBytes(it.chargeReference))
-                    _GXCommon.setData(bb, DataType.BITSTRING, it.collectionConfiguration)
+                    _GXCommon.setData(settings, bb, DataType.BITSTRING, it.collectionConfiguration)
             ret = bb.array()
         elif e.index == 12:
             bb = GXByteBuffer()
@@ -314,9 +326,9 @@ class GXDLMSAccount(GXDLMSObject, IGXDLMSBase):
             bb = GXByteBuffer()
             bb.setUInt8(DataType.STRUCTURE)
             bb.setUInt8(3)
-            _GXCommon.setData(bb, DataType.STRING_UTF8, self.currency.name)
-            _GXCommon.setData(bb, DataType.INT8, self.currency.scale)
-            _GXCommon.setData(bb, DataType.ENUM, self.currency.unit)
+            _GXCommon.setData(settings, bb, DataType.STRING_UTF8, self.currency.name)
+            _GXCommon.setData(settings, bb, DataType.INT8, self.currency.scale)
+            _GXCommon.setData(settings, bb, DataType.ENUM, self.currency.unit)
             ret = bb.array()
         elif e.index == 16:
             ret = self.lowCreditThreshold
@@ -390,7 +402,7 @@ class GXDLMSAccount(GXDLMSObject, IGXDLMSBase):
             else:
                 tmp = None
                 if isinstance(e.value, bytearray):
-                    tmp = _GXCommon.changeType(e.value, DataType.DATETIME)
+                    tmp = _GXCommon.changeType(settings, e.value, DataType.DATETIME)
                 else:
                     tmp = e.value
                 self.accountActivationTime = tmp
@@ -400,7 +412,7 @@ class GXDLMSAccount(GXDLMSObject, IGXDLMSBase):
             else:
                 tmp = None
                 if isinstance(e.value, bytearray):
-                    tmp = _GXCommon.changeType(e.value, DataType.DATETIME)
+                    tmp = _GXCommon.changeType(settings, e.value, DataType.DATETIME)
                 else:
                     tmp = e.value
                 self.accountClosureTime = tmp
@@ -464,12 +476,8 @@ class GXDLMSAccount(GXDLMSObject, IGXDLMSBase):
         self.loadReferences(reader, "ChargeReferences", self.chargeReferences)
         self.loadCreditChargeConfigurations(reader, self.creditChargeConfigurations)
         self.loadTokenGatewayConfigurations(reader, self.tokenGatewayConfigurations)
-        tmp = reader.readElementContentAsString("AccountActivationTime")
-        if tmp:
-            self.accountActivationTime = GXDateTime(tmp)
-        tmp = reader.readElementContentAsString("AccountClosureTime")
-        if tmp:
-            self.accountClosureTime = GXDateTime(tmp)
+        self.accountActivationTime = reader.readElementContentAsDateTime("AccountActivationTime")
+        self.accountClosureTime = reader.readElementContentAsDateTime("AccountClosureTime")
         self.currency.name = reader.readElementContentAsString("CurrencyName")
         self.currency.scale = reader.readElementContentAsInt("CurrencyScale")
         self.currency.unit = reader.readElementContentAsInt("CurrencyUnit")
@@ -512,11 +520,10 @@ class GXDLMSAccount(GXDLMSObject, IGXDLMSBase):
             writer.writeEndElement()
 
     def save(self, writer):
-        writer.writeElementString("PaymentMode", self.paymentMode)
-        writer.writeElementString("AccountStatus", self.accountStatus)
+        writer.writeElementString("PaymentMode", int(self.paymentMode))
+        writer.writeElementString("AccountStatus", int(self.accountStatus))
         writer.writeElementString("CurrentCreditInUse", self.currentCreditInUse)
-        if self.currentCreditStatus:
-            writer.writeElementString("CurrentCreditStatus", self.currentCreditStatus)
+        writer.writeElementString("CurrentCreditStatus", int(self.currentCreditStatus))
         writer.writeElementString("AvailableCredit", self.availableCredit)
         writer.writeElementString("AmountToClear", self.amountToClear)
         writer.writeElementString("ClearanceThreshold", self.clearanceThreshold)
@@ -525,10 +532,8 @@ class GXDLMSAccount(GXDLMSObject, IGXDLMSBase):
         self.saveReferences(writer, self.chargeReferences, "ChargeReferences")
         self.saveCreditChargeConfigurations(writer, self.creditChargeConfigurations)
         self.saveTokenGatewayConfigurations(writer, self.tokenGatewayConfigurations)
-        if self.accountActivationTime:
-            writer.writeElementString("AccountActivationTime", self.accountActivationTime.toFormatString())
-        if self.accountClosureTime:
-            writer.writeElementString("AccountClosureTime", self.accountClosureTime.toFormatString())
+        writer.writeElementString("AccountActivationTime", self.accountActivationTime)
+        writer.writeElementString("AccountClosureTime", self.accountClosureTime)
         writer.writeElementString("CurrencyName", self.currency.name)
         writer.writeElementString("CurrencyScale", self.currency.scale)
         writer.writeElementString("CurrencyUnit", int(self.currency.unit))
