@@ -116,3 +116,14 @@ from .GXXmlWriter import GXXmlWriter
 from .GXXmlWriterSettings import GXXmlWriterSettings
 from .IGXDLMSBase import IGXDLMSBase
 from .GXDLMSUtilityTables import GXDLMSUtilityTables
+from .GXDLMSLlcSscsSetup import GXDLMSLlcSscsSetup
+from .GXDLMSPrimeNbOfdmPlcPhysicalLayerCounters import GXDLMSPrimeNbOfdmPlcPhysicalLayerCounters
+from .GXDLMSPrimeNbOfdmPlcMacSetup import GXDLMSPrimeNbOfdmPlcMacSetup
+from .GXDLMSPrimeNbOfdmPlcMacFunctionalParameters import GXDLMSPrimeNbOfdmPlcMacFunctionalParameters
+from .GXDLMSPrimeNbOfdmPlcMacCounters import GXDLMSPrimeNbOfdmPlcMacCounters
+from .GXDLMSPrimeNbOfdmPlcMacNetworkAdministrationData import GXDLMSPrimeNbOfdmPlcMacNetworkAdministrationData
+from .GXDLMSPrimeNbOfdmPlcApplicationsIdentification import GXDLMSPrimeNbOfdmPlcApplicationsIdentification
+from .GXMacMulticastEntry import GXMacMulticastEntry
+from .GXMacDirectTable import GXMacDirectTable
+from .GXMacAvailableSwitch import GXMacAvailableSwitch
+from .GXMacPhyCommunication import GXMacPhyCommunication
